@@ -1191,7 +1191,8 @@ galera::Certification::append_trx(const TrxHandleSlavePtr& trx)
                       << " trx seqno " << trx->global_seqno();
         }
 
-        if (gu_unlikely((trx->last_seen_seqno() + 1) < trx_map_.begin()->first))
+        if (!trx_map_.empty() &&
+            gu_unlikely((trx->last_seen_seqno() + 1) < trx_map_.begin()->first))
         {
             /* See #733 - for now it is false positive */
             cert_debug
@@ -1289,11 +1290,16 @@ wsrep_seqno_t galera::Certification::set_trx_committed(TrxHandleSlave& trx)
         {
             assert(trx.last_seen_seqno() != WSREP_SEQNO_UNDEFINED);
             DepsSet::iterator i(deps_set_.find(trx.last_seen_seqno()));
-            assert(i != deps_set_.end());
+            if (i != deps_set_.end())
+            {
+                if (deps_set_.size() == 1) safe_to_discard_seqno_ = *i;
 
-            if (deps_set_.size() == 1) safe_to_discard_seqno_ = *i;
-
-            deps_set_.erase(i);
+                deps_set_.erase(i);
+            }
+            else
+            {
+                assert(i != deps_set_.end());
+            }
         }
 
         if (gu_unlikely(index_purge_required()))
