@@ -66,6 +66,8 @@ namespace galera
         virtual int     get_status(gu::Status&) const = 0;
         virtual void    get_membership(wsrep_allocator_cb        alloc,
                                        struct wsrep_membership** memb) const =0;
+        virtual void    get_membership(wsrep_allocator_cb        alloc,
+                                       struct wsrep_membership_v2** memb) const =0;
         virtual int     fetch_pfs_info(wsrep_node_info_t** nodes,
                                        uint32_t* size,
                                        int32_t* my_index,
@@ -277,6 +279,12 @@ namespace galera
 	    gcs_get_membership(conn_, alloc, memb);
         }
 
+        void get_membership(wsrep_allocator_cb alloc,
+                            struct wsrep_membership_v2** memb) const
+        {
+            gcs_get_membership(conn_, alloc, memb);
+        }
+
         void param_set (const std::string& key, const std::string& value)
         {
             long ret = gcs_param_set (conn_, key.c_str(), value.c_str());
@@ -472,6 +480,12 @@ namespace galera
 
         void get_membership(wsrep_allocator_cb        alloc,
                             struct wsrep_membership** memb) const
+        {
+            *memb = 0;
+        }
+
+        void get_membership(wsrep_allocator_cb        alloc,
+                            struct wsrep_membership_v2** memb) const
         {
 	    *memb = 0;
         }
