@@ -88,6 +88,14 @@ ReplicatorSMM::sst_received(const wsrep_gtid_t& state_id,
         return WSREP_CONN_FAIL;
     }
 
+    if (rcode == -ECANCELED) {
+        // perform a graceful shutdown
+        sst_graceful_shutdown_ = true;
+        sst_received_ = true;
+        sst_cond_.signal();
+        return WSREP_OK;
+    }
+
     assert(rcode <= 0);
     if (rcode) { assert(state_id.seqno == WSREP_SEQNO_UNDEFINED); }
 
@@ -193,7 +201,7 @@ StateRequest_v1::StateRequest_v1 (
                                << ") unrepresentable";
 
     if (ist_req_len > INT32_MAX || ist_req_len < 0)
-        gu_throw_error (EMSGSIZE) << "IST request length (" << sst_req_len
+        gu_throw_error (EMSGSIZE) << "IST request length (" << ist_req_len
                                << ") unrepresentable";
 
     char* ptr(req_);
@@ -930,6 +938,19 @@ ReplicatorSMM::request_state_transfer (void* recv_ctx,
             while (false == sst_received_) sst_lock.wait(sst_cond_);
         }
 
+        if (sst_graceful_shutdown_) {
+            log_warn  << "State transfer interrupted, shutting down gracefully:"
+                      << "\n\t sst_uuid = " << sst_uuid_
+                      << "\n\t sst_seqno = " << sst_seqno_
+                      << "\n\t group_uuid = " << group_uuid
+                      << "\n\t safe_to_bootstrap_ = " << safe_to_bootstrap_
+                      << "\n\t cc_seqno = " << cc_seqno;
+
+            st_.restore_saved_state();
+
+            abort();
+        }
+
         if (sst_uuid_ != group_uuid)
         {
             log_fatal << "Application received wrong state: "
@@ -1075,8 +1096,7 @@ ReplicatorSMM::request_state_transfer (void* recv_ctx,
                 // Note: apply_monitor_ must be drained to avoid race between
                 // IST appliers and GCS appliers, GCS action source may
                 // provide actions that have already been applied via IST.
-                log_info << "Draining apply monitors after IST up to "
-                         << sst_seqno_;
+                log_info << "Draining apply monitors after IST up to " << sst_seqno_;
                 apply_monitor_.drain(sst_seqno_);
                 set_initial_position(group_uuid, sst_seqno_);
             }
@@ -1484,7 +1504,7 @@ void ReplicatorSMM::ist_cc(const gcs_act_cchange& conf,
             establish_protocol_versions(conf.repl_proto_ver);
             cert_.adjust_position(*view_info, gu::GTID(conf.uuid, conf.seqno),
                                   trx_params_.version_);
-            // record CC related state seqnos, needed for IST on DONOR
+            // record CC related state seqnos needed for IST on DONOR
             record_cc_seqnos(conf.seqno, "preload");
             ::free(view_info);
         }
